@@ -1,4 +1,4 @@
-# Copyright (c) 2012-2014, Eucalyptus Systems, Inc.
+# Copyright (c) 2012-2016 Hewlett Packard Enterprise Development LP
 #
 # Permission to use, copy, modify, and/or distribute this software for
 # any purpose with or without fee is hereby granted, provided that the
@@ -15,6 +15,7 @@
 from __future__ import absolute_import
 
 import argparse
+import copy
 from functools import partial
 import logging
 import os.path
@@ -65,24 +66,22 @@ class BaseRequest(BaseCommand):
     '''
 
     SERVICE_CLASS = BaseService
-    AUTH_CLASS    = None
-    NAME          = None
-    METHOD        = 'GET'
+    AUTH_CLASS = None
+    NAME = None
+    METHOD = 'GET'
 
     DEFAULT_ROUTES = (PARAMS,)
-    LIST_TAGS = []
-
 
     def __init__(self, service=None, auth=None, **kwargs):
         self.auth = auth
         self.service = service
         # Parts of the HTTP request to be sent to the server.
-        self.method    = self.METHOD
-        self.path      = None
-        self.headers   = {}
-        self.params    = {}
-        self.body      = ''
-        self.files     = {}
+        self.method = self.METHOD
+        self.path = None
+        self.headers = {}
+        self.params = {}
+        self.body = ''
+        self.files = {}
 
         # HTTP response obtained from the server
         self.response = None
@@ -95,18 +94,10 @@ class BaseRequest(BaseCommand):
         if self.service is None and self.SERVICE_CLASS is not None:
             self.service = self.SERVICE_CLASS(self.config,
                                               loglevel=self.log.level)
-        if self.auth is None:
-            if self.AUTH_CLASS is not None:
-                self.auth = self.AUTH_CLASS(self.config,
-                                            loglevel=self.log.level)
-            elif self.SERVICE_CLASS.AUTH_CLASS is not None:
-                # Backward compatibility
-                msg = ('BaseService.AUTH_CLASS is deprecated; use '
-                       'BaseRequest.AUTH_CLASS instead')
-                self.log.warn(msg)
-                warnings.warn(msg, DeprecationWarning)
-                self.auth = self.SERVICE_CLASS.AUTH_CLASS(
-                    self.config, loglevel=self.log.level)
+        if self.auth is None and self.AUTH_CLASS is not None:
+            # pylint: disable=not-callable
+            self.auth = self.AUTH_CLASS(self.config, loglevel=self.log.level)
+            # pylint: enable=not-callable
         BaseCommand._post_init(self)
 
     @classmethod
@@ -149,6 +140,9 @@ class BaseRequest(BaseCommand):
 
     @property
     def status(self):
+        msg = 'BaseRequest.status is deprecated, use response status'
+        self.log.warn(msg)
+        warnings.warn(msg, DeprecationWarning)  # deprecated in 0.3
         if self.response is not None:
             return self.response.status
         else:
@@ -159,11 +153,10 @@ class BaseRequest(BaseCommand):
             self.log.warn('send() called before configure(); bugs may result')
         headers = dict(self.headers or {})
         headers.setdefault('User-Agent', self.suite.get_user_agent())
-        params  = self.prepare_params()
         try:
             self.response = self.service.send_request(
                 method=self.method, path=self.path, headers=headers,
-                params=params, data=self.body, files=self.files,
+                params=self.params, data=self.body, files=self.files,
                 auth=self.auth)
             return self.parse_response(self.response)
         except ServerError as err:
@@ -180,16 +173,12 @@ class BaseRequest(BaseCommand):
             return self.handle_server_error(err)
 
     def handle_server_error(self, err):
-        self.log.debug('-- response content --\n',
-                       extra={'append': True})
+        self.log.debug('-- response content --\n', extra={'append': True})
         self.log.debug(self.response.text, extra={'append': True})
         self.log.debug('-- end of response content --')
         self.log.info('result: failure')
         raise
 
-    def prepare_params(self):
-        return self.params or {}
-
     def parse_response(self, response):
         return response
 
@@ -240,14 +229,16 @@ class BaseRequest(BaseCommand):
 class AWSQueryRequest(BaseRequest):
     API_VERSION = None
     FILTERS = []
+    LIST_TAGS = []
 
-    def populate_parser(self, parser, arg_objs):
-        BaseRequest.populate_parser(self, parser, arg_objs)
+    def _populate_parser(self, parser, arg_objs):
+        BaseRequest._populate_parser(self, parser, arg_objs)
         if self.FILTERS:
-            parser.add_argument('--filter', metavar='NAME=VALUE',
-                    action='append', dest='filters',
-                    help='restrict results to those that meet criteria',
-                    type=partial(_parse_filter, filter_objs=self.FILTERS))
+            parser.add_argument(
+                '--filter', metavar='NAME=VALUE',
+                action='append', dest='filters',
+                help='restrict results to those that meet criteria',
+                type=partial(_parse_filter, filter_objs=self.FILTERS))
             parser.epilog = self.__build_filter_help()
             self._arg_routes['filters'] = (None,)
 
@@ -261,7 +252,9 @@ class AWSQueryRequest(BaseRequest):
     def action(self):
         return self.name
 
-    def prepare_params(self):
+    def send(self):
+        orig_params = self.params
+        params = copy.deepcopy(self.params)
         params = self.flatten_params(self.params)
         params['Action'] = self.action
         params['Version'] = self.API_VERSION or self.service.API_VERSION
@@ -270,15 +263,26 @@ class AWSQueryRequest(BaseRequest):
             if key.lower().endswith('password'):
                 # This makes it slightly more obvious that this is redacted by
                 # the framework and not just a string.
-                redacted_params[key] = type('REDACTED', (),
-                        {'__repr__': lambda self: '<redacted>'})()
+                redacted_params[key] = type(
+                    'REDACTED', (), {'__repr__': lambda self: '<redacted>'})()
         self.log.info('parameters: %s', redacted_params)
-        return params
+
+        if self.method.upper() == 'POST':
+            self.log.debug('sending flattened parameters as form data')
+            self.body = params
+            self.params = {}
+        else:
+            self.log.debug('sending flattened parameters as query string')
+            self.params = params
+        try:
+            return BaseRequest.send(self)
+        finally:
+            self.params = orig_params
 
     def parse_response(self, response):
         # Parser for list-delimited responses like EC2's
-        response_dict = self.log_and_parse_response(response,
-                parse_listdelimited_aws_xml, list_tags=self.LIST_TAGS)
+        response_dict = self.log_and_parse_response(
+            response, parse_listdelimited_aws_xml, list_tags=self.LIST_TAGS)
         # Strip off the root element
         assert len(response_dict) == 1
         return response_dict[list(response_dict.keys())[0]]
@@ -334,6 +338,8 @@ class AWSQueryRequest(BaseRequest):
                     flattened[prefixed_key] = str(val).lower()
                 elif isinstance(val, file):
                     flattened[prefixed_key] = val.read()
+                elif isinstance(val, float):
+                    flattened[prefixed_key] = str(val)
                 elif val or val is 0:
                     flattened[prefixed_key] = str(val)
                 elif val is EMPTY:
@@ -373,7 +379,9 @@ class AWSQueryRequest(BaseRequest):
 
         helplines = ['allowed filter names:']
         for filter_obj in self.FILTERS:
-            if filter_obj.help:
+            if filter_obj.help == argparse.SUPPRESS:
+                continue
+            elif filter_obj.help:
                 first, _, rest = filter_obj.help.partition('\n')
                 if rest.startswith(' ') and not first.startswith(' '):
                     # First line is not uniformly indented
@@ -383,13 +391,14 @@ class AWSQueryRequest(BaseRequest):
                 if len(filter_obj.name) <= 20:
                     # Short name; start on same line and pad two spaces
                     firstline = '  {0:<20}  '.format(filter_obj.name)
-                    wrapper = textwrap.TextWrapper(fix_sentence_endings=True,
-                        initial_indent=firstline, subsequent_indent=(' ' * 24))
+                    wrapper = textwrap.TextWrapper(
+                        fix_sentence_endings=True, initial_indent=firstline,
+                        subsequent_indent=(' ' * 24))
                 else:
                     # Long name; start on next line
                     helplines.append('  ' + filter_obj.name)
-                    wrapper = textwrap.TextWrapper(fix_sentence_endings=True,
-                        initial_indent=(' ' * 24),
+                    wrapper = textwrap.TextWrapper(
+                        fix_sentence_endings=True, initial_indent=(' ' * 24),
                         subsequent_indent=(' ' * 24))
                 helplines.extend(wrapper.wrap(content))
             else:
@@ -433,10 +442,10 @@ def _process_filters(cli_filters):
 
 class _IteratorFileObjAdapter(object):
     def __init__(self, source):
-        self._source  = source
+        self._source = source
         self._buflist = []
-        self._closed  = False
-        self._len     = 0
+        self._closed = False
+        self._len = 0
 
     def __enter__(self):
         return self
@@ -459,7 +468,7 @@ class _IteratorFileObjAdapter(object):
                 self._buflist.append(chunk)
             result = ''.join(self._buflist)
             self._buflist = []
-            self._len     = 0
+            self._len = 0
         else:
             while self._len < size:
                 try:
@@ -468,13 +477,13 @@ class _IteratorFileObjAdapter(object):
                     self._len += len(chunk)
                 except StopIteration:
                     break
-            result    = ''.join(self._buflist)
+            result = ''.join(self._buflist)
             extra_len = len(result) - size
             self._buflist = []
-            self._len     = 0
+            self._len = 0
             if extra_len > 0:
                 self._buflist = [result[-extra_len:]]
-                self._len     = extra_len
+                self._len = extra_len
                 result = result[:-extra_len]
         return result
 
@@ -482,8 +491,8 @@ class _IteratorFileObjAdapter(object):
 class _ReadLoggingFileWrapper(object):
     def __init__(self, fileobj, logger, level):
         self.fileobj = fileobj
-        self.logger  = logger
-        self.level   = level
+        self.logger = logger
+        self.level = level
 
     def read(self, size=-1):
         chunk = self.fileobj.read(size)
